pro IRIS_level1to2, level1dir

  ;Assumption: all FUV/NUV have the same regions (=only one CRS-file for each)
  ;PZTY is ignored
  ;if spectral/spatial summation is greater than 1, there might be a problem with the row and column numbers
  ;
  ;time, PZTX, PZTY are saved in the last extension as an array as follows [rasterstep, 3]
  ;where [*,0]=time, [*,1]=PZTX, [*,2]=PZTY
  ;
  ;
  ;INPUT
  ;level1dir: directory of level 1 files


  if N_PARAMS() eq 0 then begin
    ;level1dir = 'OLD/2012_09_26_08_30_36/'  ;directory of level 1 files
    cd, current=cur_dir
    level1dir = dialog_pickfile(/directory, path=cur_dir, title='Please select a directory, which contains the Level 1 data')
  endif
  ;level2dir = '2012_09_26_08_30_36_l2/'  ;directory of level 2 files
  
  
  ;get file lists
  fuvfiles = file_search(level1dir+'*_FUV_L1.fits')
  Nfuvfiles = N_ELEMENTS(fuvfiles)
  nuvfiles = file_search(level1dir+'*_NUV_L1.fits')
  Nnuvfiles = N_ELEMENTS(nuvfiles)
  sjifiles = file_search(level1dir+'*_SJI_L1.fits')
  Nsjifiles = N_ELEMENTS(sjifiles)
  
  if Nfuvfiles+Nnuvfiles+Nsjifiles le 3 then begin
  stop
    if (fuvfiles eq '') && (nuvfiles eq '') && (sjifiles eq '') then begin
      print, 'No files found in this directory'
      print, level1dir
      return
    endif
  endif
  
  lastpos = STRPOS(level1dir, '_L1', /reverse_search)
  if lastpos eq -1 then begin
    level2dir = STRMID(level1dir, 0, STRLEN(level1dir)-1) + '_L2/'
  endif else begin
    level2dir = level1dir
    STRPUT, level2dir, '_L2', lastpos
  endelse
  print, 'saving data in folder: ', level2dir
  
  FILE_MKDIR, level2dir



  return
  
  
  ;FUV
  
  ;get number of rasters, and their start and end file
  startrasterfuv=[0]
  for filenr=0,N_ELEMENTS(fuvfiles)-1 do begin
    fxread, fuvfiles[filenr], data, header, /nodata
    pztx = fxpar(header, 'PZTX')
    
    if filenr gt 0 then begin
      dpztx = pztx - pztxold
      if dpztx ne 0 then dpztx=dpztx/abs(dpztx)
      if dpztxold ne 9 then begin ;if this is not the second take of a raster
        if dpztx ne dpztxold then begin ;raster ends, start a new one
          if N_ELEMENTS(startrasterfuv) eq 1 then endrasterfuv = filenr-1 $
          else endrasterfuv = [endrasterfuv, filenr-1]
          startrasterfuv = [startrasterfuv, filenr]
          dpztxold = 9
        endif
      endif else dpztxold=dpztx
    endif else begin
      dpztxold = 9
    endelse
    pztxold = pztx
  endfor ;fuvfiles
  endrasterfuv = [endrasterfuv, filenr-1]
  
  print, 'found ' + string(N_ELEMENTS(startrasterfuv)) + ' rasters'
  print, startrasterfuv
  print, endrasterfuv
  
  
  OBSid = fxpar(header, 'OBSID')
  FDBid = fxpar(header, 'FDBID')
  CRSid = fxpar(header, 'CRSID')
  
  numregions = fxpar(header, 'REGTOT')
  startrow = fxpar(header, 'ROWSTAR*')
  numrows = fxpar(header, 'ROWNUM*')
  startcol = fxpar(header, 'COLSTAR*')
  numcols = fxpar(header, 'COLNUM*')
  rasterdata = ptrarr(numregions)
  
  for raster=0,N_ELEMENTS(startrasterfuv)-1 do begin
    numsteps = endrasterfuv[raster] - startrasterfuv[raster] + 1
    for i=0,numregions-1 do rasterdata[i] = ptr_new(fltarr(numrows[i], numcols[i], numsteps))
    auxdata = fltarr(numsteps, 3)
    
    for filenr=0,numsteps-1 do begin
      fxread, fuvfiles[filenr+startrasterfuv[raster]], data, header
      auxdata[filenr,0] = fxpar(header, 'TIME')
      auxdata[filenr,1] = fxpar(header, 'PZTX')
      auxdata[filenr,2] = fxpar(header, 'PZTY')
      
      for region=0,numregions-1 do $
        (*rasterdata[region])[*,*,filenr] = data[startrow[region]:startrow[region]+numrows[region]-1, startcol[region]:startcol[region]+numcols[region]-1]
        
    endfor ;filenr
    
    ;we have a new raster, now we save it into a fits file
    file2=level2dir+fns('FUV_L2_####.fits',raster)
    for region=0,numregions-1 do begin
      if region eq 0 then begin
        ;first region is written into primary block
        mkhdr, header, (*rasterdata[region]), /extend
        ;add some information to the primary header
        sxaddpar, header, 'OBSID', OBSid, 'OBS List ID'
        sxaddpar, header, 'FDBID', FDBid, 'FDB List ID'
        sxaddpar, header, 'CRSID', CRSid, 'CRS List ID'
        sxaddpar, header, 'REGTOT', numregions, 'Number of Regions'
        sxaddpar, header, 'COMMENT', 'Matrix: [wavelength, y, t/x]'
        sxaddpar, header, 'COMMENT', 'There is an additional extension with auxiliary data'
      endif else $
        mkhdr, header, (*rasterdata[region]), /image
        
      ;add information to all headers
      sxaddpar, header, 'ROWSTAR', startrow[region], 'Startrow'
      sxaddpar, header, 'ROWNUM', numrows[region], 'Number of Rows'
      sxaddpar, header, 'COLSTAR', startcol[region], 'Startcolumn'
      sxaddpar, header, 'COLNUM', numcols[region], 'Number of Columns'
      sxaddpar, header, 'DESC', 'Some line', 'The name of the spectral line in this region (TO DO)'
      
      if region eq 0 then $
        writefits, file2, (*rasterdata[region]), header $
      else $
        writefits, file2, (*rasterdata[region]), header, /append
        
    endfor ;regions
    
    ;write last extension with auxiliary data (time, PZTX, PZTY)
    mkhdr, header, auxdata, /image
    sxaddpar, header, 'COMMENT', 'Auxiliary data'
    sxaddpar, header, 'COMMENT', '[*,0]=time, [*,1]=PZTX, [*,2]=PZTY'
    writefits, file2, auxdata, header, /append
  endfor ;raster
  
  
  
  
  
  
  
  
  
  ;NUV
  
  
  ;get number of rasters, and their start and end file
  startrasternuv=[0]
  for filenr=0,N_ELEMENTS(nuvfiles)-1 do begin
    fxread, nuvfiles[filenr], data, header, /nodata
    pztx = fxpar(header, 'PZTX')
    
    if filenr gt 0 then begin
      dpztx = pztx - pztxold
      if dpztx ne 0 then dpztx=dpztx/abs(dpztx)
      if dpztxold ne 9 then begin ;if this is not the second take of a raster
        if dpztx ne dpztxold then begin ;raster ends, start a new one
          if N_ELEMENTS(startrasternuv) eq 1 then endrasternuv = filenr-1 $
          else endrasternuv = [endrasternuv, filenr-1]
          startrasternuv = [startrasternuv, filenr]
          dpztxold = 9
        endif
      endif else dpztxold=dpztx
    endif else begin
      dpztxold = 9
    endelse
    pztxold = pztx
  endfor ;nuvfiles
  endrasternuv = [endrasternuv, filenr-1]
  
  print, 'found ' + string(N_ELEMENTS(startrasternuv)) + ' rasters'
  print, startrasternuv
  print, endrasternuv
  
  
  OBSid = fxpar(header, 'OBSID')
  FDBid = fxpar(header, 'FDBID')
  CRSid = fxpar(header, 'CRSID')
  
  numregions = fxpar(header, 'REGTOT')
  startrow = fxpar(header, 'ROWSTAR*')
  numrows = fxpar(header, 'ROWNUM*')
  startcol = fxpar(header, 'COLSTAR*')
  numcols = fxpar(header, 'COLNUM*')
  rasterdata = ptrarr(numregions)
  
  for raster=0,N_ELEMENTS(startrasternuv)-1 do begin
    numsteps = endrasternuv[raster] - startrasternuv[raster] + 1
    for i=0,numregions-1 do rasterdata[i] = ptr_new(fltarr(numrows[i], numcols[i], numsteps))
    auxdata = fltarr(numsteps, 3)
    
    for filenr=0,numsteps-1 do begin
      fxread, nuvfiles[filenr+startrasternuv[raster]], data, header
      auxdata[filenr,0] = fxpar(header, 'TIME')
      auxdata[filenr,1] = fxpar(header, 'PZTX')
      auxdata[filenr,2] = fxpar(header, 'PZTY')
      
      for region=0,numregions-1 do $
        (*rasterdata[region])[*,*,filenr] = data[startrow[region]:startrow[region]+numrows[region]-1, startcol[region]:startcol[region]+numcols[region]-1]
        
    endfor ;filenr
    
    ;we have a new raster, now we save it into a fits file
    file2=level2dir+fns('NUV_L2_####.fits',raster)
    for region=0,numregions-1 do begin
      if region eq 0 then begin
        ;first region is written into primary block
        mkhdr, header, (*rasterdata[region]), /extend
        ;add some information to the primary header
        sxaddpar, header, 'OBSID', OBSid, 'OBS List ID'
        sxaddpar, header, 'FDBID', FDBid, 'FDB List ID'
        sxaddpar, header, 'CRSID', CRSid, 'CRS List ID'
        sxaddpar, header, 'REGTOT', numregions, 'Number of Regions'
        sxaddpar, header, 'COMMENT', 'Matrix: [wavelength, y, t/x]'
        sxaddpar, header, 'COMMENT', 'There is an additional extension with auxiliary data'
      endif else $
        mkhdr, header, (*rasterdata[region]), /image
        
      ;add information to all headers
      sxaddpar, header, 'ROWSTAR', startrow[region], 'Startrow'
      sxaddpar, header, 'ROWNUM', numrows[region], 'Number of Rows'
      sxaddpar, header, 'COLSTAR', startcol[region], 'Startcolumn'
      sxaddpar, header, 'COLNUM', numcols[region], 'Number of Columns'
      sxaddpar, header, 'DESC', 'Some line', 'The name of the spectral line in this region (TO DO)'
      
      if region eq 0 then $
        writefits, file2, (*rasterdata[region]), header $
      else $
        writefits, file2, (*rasterdata[region]), header, /append
        
    endfor ;regions
    
    ;write last extension with auxiliary data (time, PZTX, PZTY)
    mkhdr, header, auxdata, /image
    sxaddpar, header, 'COMMENT', 'Auxiliary data'
    sxaddpar, header, 'COMMENT', '[*,0]=time, [*,1]=PZTX, [*,2]=PZTY'
    writefits, file2, auxdata, header, /append
  endfor ;raster
  
end
